@@ -4,7 +4,7 @@ import json
 import aiopg
 
 from ..connection import TIMEOUT
-from ..utils import _ContextManager, _PoolContextManager, get_running_loop
+from ..utils import _ContextManager, get_running_loop
 from .connection import SAConnection
 
 try:
@@ -13,7 +13,7 @@ try:
         PGDialect_psycopg2,
     )
 except ImportError:  # pragma: no cover
-    raise ImportError('aiopg.sa requires sqlalchemy')
+    raise ImportError("aiopg.sa requires sqlalchemy")
 
 
 class APGCompiler_psycopg2(PGCompiler_psycopg2):
@@ -33,8 +33,9 @@ class APGCompiler_psycopg2(PGCompiler_psycopg2):
 
 
 def get_dialect(json_serializer=json.dumps, json_deserializer=lambda x: x):
-    dialect = PGDialect_psycopg2(json_serializer=json_serializer,
-                                 json_deserializer=json_deserializer)
+    dialect = PGDialect_psycopg2(
+        json_serializer=json_serializer, json_deserializer=json_deserializer
+    )
 
     dialect.statement_compiler = APGCompiler_psycopg2
     dialect.implicit_returning = True
@@ -50,8 +51,16 @@ def get_dialect(json_serializer=json.dumps, json_deserializer=lambda x: x):
 _dialect = get_dialect()
 
 
-def create_engine(dsn=None, *, minsize=1, maxsize=10, dialect=_dialect,
-                  timeout=TIMEOUT, pool_recycle=-1, **kwargs):
+def create_engine(
+    dsn=None,
+    *,
+    minsize=1,
+    maxsize=10,
+    dialect=_dialect,
+    timeout=TIMEOUT,
+    pool_recycle=-1,
+    **kwargs
+):
     """A coroutine for Engine creation.
 
     Returns Engine instance with embedded connection pool.
@@ -59,18 +68,36 @@ def create_engine(dsn=None, *, minsize=1, maxsize=10, dialect=_dialect,
     The pool has *minsize* opened connections to PostgreSQL server.
     """
 
-    coro = _create_engine(dsn=dsn, minsize=minsize, maxsize=maxsize,
-                          dialect=dialect, timeout=timeout,
-                          pool_recycle=pool_recycle, **kwargs)
-    return _EngineContextManager(coro)
+    coro = _create_engine(
+        dsn=dsn,
+        minsize=minsize,
+        maxsize=maxsize,
+        dialect=dialect,
+        timeout=timeout,
+        pool_recycle=pool_recycle,
+        **kwargs
+    )
+    return _ContextManager(coro, _close_engine)
 
 
-async def _create_engine(dsn=None, *, minsize=1, maxsize=10, dialect=_dialect,
-                         timeout=TIMEOUT, pool_recycle=-1, **kwargs):
+async def _create_engine(
+    dsn=None,
+    *,
+    minsize=1,
+    maxsize=10,
+    dialect=_dialect,
+    timeout=TIMEOUT,
+    pool_recycle=-1,
+    **kwargs
+):
 
     pool = await aiopg.create_pool(
-        dsn, minsize=minsize, maxsize=maxsize,
-        timeout=timeout, pool_recycle=pool_recycle, **kwargs
+        dsn,
+        minsize=minsize,
+        maxsize=maxsize,
+        timeout=timeout,
+        pool_recycle=pool_recycle,
+        **kwargs
     )
     conn = await pool.acquire()
     try:
@@ -80,6 +107,15 @@ async def _create_engine(dsn=None, *, minsize=1, maxsize=10, dialect=_dialect,
         await pool.release(conn)
 
 
+async def _close_engine(engine: "Engine") -> None:
+    engine.close()
+    await engine.wait_closed()
+
+
+async def _close_connection(c: SAConnection) -> None:
+    await c.close()
+
+
 class Engine:
     """Connects a aiopg.Pool and
     sqlalchemy.engine.interfaces.Dialect together to provide a
@@ -164,20 +200,19 @@ class Engine:
     def acquire(self):
         """Get a connection from pool."""
         coro = self._acquire()
-        return _EngineAcquireContextManager(coro)
+        return _ContextManager[SAConnection](coro, _close_connection)
 
     async def _acquire(self):
         raw = await self._pool.acquire()
-        conn = SAConnection(raw, self)
-        return conn
+        return SAConnection(raw, self)
 
     def release(self, conn):
-        fut = self._pool.release(conn.connection)
-        return fut
+        return self._pool.release(conn.connection)
 
     def __enter__(self):
         raise RuntimeError(
-            '"await" should be used as context manager expression')
+            '"await" should be used as context manager expression'
+        )
 
     def __exit__(self, *args):
         # This must exist because __enter__ exists, even though that
@@ -208,10 +243,6 @@ class Engine:
         await self.wait_closed()
 
 
-_EngineContextManager = _PoolContextManager
-_EngineAcquireContextManager = _ContextManager
-
-
 class _ConnectionContextManager:
     """Context manager.
 
@@ -227,7 +258,7 @@ class _ConnectionContextManager:
             <block>
     """
 
-    __slots__ = ('_conn', '_loop')
+    __slots__ = ("_conn", "_loop")
 
     def __init__(self, conn: SAConnection, loop: asyncio.AbstractEventLoop):
         self._conn = conn
